@@ -1253,12 +1253,12 @@ nsPresContext::BidiEnabledExternal() const
 }
 
 void
-nsPresContext::SetBidiEnabled(PRBool aBidiEnabled) const
+nsPresContext::SetBidiEnabled() const
 {
   if (mShell) {
     nsIDocument *doc = mShell->GetDocument();
     if (doc) {
-      doc->SetBidiEnabled(aBidiEnabled);
+      doc->SetBidiEnabled();
     }
   }
 }
@@ -1286,7 +1286,7 @@ nsPresContext::SetBidi(PRUint32 aSource, PRBool aForceRestyle)
   Document()->SetBidiOptions(aSource);
   if (IBMBIDI_TEXTDIRECTION_RTL == GET_BIDI_OPTION_DIRECTION(aSource)
       || IBMBIDI_NUMERAL_HINDI == GET_BIDI_OPTION_NUMERAL(aSource)) {
-    SetBidiEnabled(PR_TRUE);
+    SetBidiEnabled();
   }
   if (IBMBIDI_TEXTTYPE_VISUAL == GET_BIDI_OPTION_TEXTTYPE(aSource)) {
     SetVisualMode(PR_TRUE);
@@ -1501,6 +1501,8 @@ nsPresContext::IsChrome() const
 /* virtual */ PRBool
 nsPresContext::HasAuthorSpecifiedRules(nsIFrame *aFrame, PRUint32 ruleTypeMask) const
 {
-  return nsRuleNode::
-    HasAuthorSpecifiedRules(aFrame->GetStyleContext(), ruleTypeMask);
+  return
+    UseDocumentColors() &&
+    nsRuleNode::HasAuthorSpecifiedRules(aFrame->GetStyleContext(),
+                                        ruleTypeMask);
 }
